@@ -1,4 +1,4 @@
-/* $OpenBSD: ciphers.c,v 1.9 2018/02/07 05:47:55 jsing Exp $ */
+/* $OpenBSD: ciphers.c,v 1.15 2022/07/19 20:15:19 tb Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -26,10 +26,12 @@
 
 struct {
 	int usage;
+	int use_supported;
 	int verbose;
+	int version;
 } ciphers_config;
 
-struct option ciphers_options[] = {
+static const struct option ciphers_options[] = {
 	{
 		.name = "h",
 		.type = OPTION_FLAG,
@@ -40,10 +42,39 @@ struct option ciphers_options[] = {
 		.type = OPTION_FLAG,
 		.opt.flag = &ciphers_config.usage,
 	},
+	{
+		.name = "s",
+		.desc = "Only list ciphers that are supported by the TLS method",
+		.type = OPTION_FLAG,
+		.opt.flag = &ciphers_config.use_supported,
+	},
 	{
 		.name = "tls1",
-		.desc = "This option is deprecated since it is the default",
-		.type = OPTION_DISCARD,
+		.desc = "Use TLS protocol version 1",
+		.type = OPTION_VALUE,
+		.opt.value = &ciphers_config.version,
+		.value = TLS1_VERSION,
+	},
+	{
+		.name = "tls1_1",
+		.desc = "Use TLS protocol version 1.1",
+		.type = OPTION_VALUE,
+		.opt.value = &ciphers_config.version,
+		.value = TLS1_1_VERSION,
+	},
+	{
+		.name = "tls1_2",
+		.desc = "Use TLS protocol version 1.2",
+		.type = OPTION_VALUE,
+		.opt.value = &ciphers_config.version,
+		.value = TLS1_2_VERSION,
+	},
+	{
+		.name = "tls1_3",
+		.desc = "Use TLS protocol version 1.3",
+		.type = OPTION_VALUE,
+		.opt.value = &ciphers_config.version,
+		.value = TLS1_3_VERSION,
 	},
 	{
 		.name = "v",
@@ -65,7 +96,8 @@ struct option ciphers_options[] = {
 static void
 ciphers_usage(void)
 {
-	fprintf(stderr, "usage: ciphers [-hVv] [-tls1] [cipherlist]\n");
+	fprintf(stderr, "usage: ciphers [-hsVv] [-tls1] [-tls1_1] [-tls1_2] "
+	    "[-tls1_3] [cipherlist]\n");
 	options_usage(ciphers_options);
 }
 
@@ -74,6 +106,7 @@ ciphers_main(int argc, char **argv)
 {
 	char *cipherlist = NULL;
 	STACK_OF(SSL_CIPHER) *ciphers;
+	STACK_OF(SSL_CIPHER) *supported_ciphers = NULL;
 	const SSL_CIPHER *cipher;
 	SSL_CTX *ssl_ctx = NULL;
 	SSL *ssl = NULL;
@@ -101,9 +134,18 @@ ciphers_main(int argc, char **argv)
 		return (1);
 	}
 
-	if ((ssl_ctx = SSL_CTX_new(TLSv1_client_method())) == NULL)
+	if ((ssl_ctx = SSL_CTX_new(TLS_method())) == NULL)
 		goto err;
 
+	if (ciphers_config.version != 0) {
+		if (!SSL_CTX_set_min_proto_version(ssl_ctx,
+		    ciphers_config.version))
+			goto err;
+		if (!SSL_CTX_set_max_proto_version(ssl_ctx,
+		    ciphers_config.version))
+			goto err;
+	}
+
 	if (cipherlist != NULL) {
 		if (SSL_CTX_set_cipher_list(ssl_ctx, cipherlist) == 0)
 			goto err;
@@ -112,8 +154,15 @@ ciphers_main(int argc, char **argv)
 	if ((ssl = SSL_new(ssl_ctx)) == NULL)
 		goto err;
 
-	if ((ciphers = SSL_get_ciphers(ssl)) == NULL)
-		goto err;
+	if (ciphers_config.use_supported) {
+		if ((supported_ciphers =
+		    SSL_get1_supported_ciphers(ssl)) == NULL)
+			goto err;
+		ciphers = supported_ciphers;
+	} else {
+		if ((ciphers = SSL_get_ciphers(ssl)) == NULL)
+			goto err;
+	}
 
 	for (i = 0; i < sk_SSL_CIPHER_num(ciphers); i++) {
 		cipher = sk_SSL_CIPHER_value(ciphers, i);
@@ -125,7 +174,7 @@ ciphers_main(int argc, char **argv)
 		if (ciphers_config.verbose > 1) {
 			value = SSL_CIPHER_get_value(cipher);
 			fprintf(stdout, "%-*s0x%02X,0x%02X - ", 10, "",
-				((value >> 8) & 0xff), (value & 0xff));
+			    ((value >> 8) & 0xff), (value & 0xff));
 		}
 		desc = SSL_CIPHER_description(cipher, NULL, 0);
 		if (strcmp(desc, "OPENSSL_malloc Error") == 0) {
@@ -145,6 +194,7 @@ ciphers_main(int argc, char **argv)
 	rv = 1;
 
  done:
+	sk_SSL_CIPHER_free(supported_ciphers);
 	SSL_CTX_free(ssl_ctx);
 	SSL_free(ssl);
 
