@@ -1,4 +1,4 @@
-/* $OpenBSD: p12_kiss.c,v 1.21 2021/07/09 14:08:00 tb Exp $ */
+/* $OpenBSD: p12_kiss.c,v 1.25 2022/08/20 09:16:18 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 1999.
  */
@@ -61,6 +61,8 @@
 #include <openssl/err.h>
 #include <openssl/pkcs12.h>
 
+#include "pkcs12_local.h"
+
 /* Simplified PKCS#12 routines */
 
 static int parse_pk12( PKCS12 *p12, const char *pass, int passlen,
@@ -84,18 +86,17 @@ PKCS12_parse(PKCS12 *p12, const char *pass, EVP_PKEY **pkey, X509 **cert,
 {
 	STACK_OF(X509) *ocerts = NULL;
 	X509 *x = NULL;
-	/* Check for NULL PKCS12 structure */
 
-	if (!p12) {
-		PKCS12error(PKCS12_R_INVALID_NULL_PKCS12_POINTER);
-		return 0;
-	}
-
-	if (pkey)
+	if (pkey != NULL)
 		*pkey = NULL;
-	if (cert)
+	if (cert != NULL)
 		*cert = NULL;
 
+	if (p12 == NULL) {
+		PKCS12error(PKCS12_R_INVALID_NULL_PKCS12_POINTER);
+		goto err;
+	}
+
 	/* Check the mac */
 
 	/* If password is zero length or NULL then try verifying both cases
@@ -104,7 +105,7 @@ PKCS12_parse(PKCS12 *p12, const char *pass, EVP_PKEY **pkey, X509 **cert,
 	 * password are two different things...
 	 */
 
-	if (!pass || !*pass) {
+	if (pass == NULL || *pass == '\0') {
 		if (PKCS12_verify_mac(p12, NULL, 0))
 			pass = NULL;
 		else if (PKCS12_verify_mac(p12, "", 0))
@@ -119,10 +120,9 @@ PKCS12_parse(PKCS12 *p12, const char *pass, EVP_PKEY **pkey, X509 **cert,
 	}
 
 	/* Allocate stack for other certificates */
-	ocerts = sk_X509_new_null();
-	if (!ocerts) {
+	if ((ocerts = sk_X509_new_null()) == NULL) {
 		PKCS12error(ERR_R_MALLOC_FAILURE);
-		return 0;
+		goto err;
 	}
 
 	if (!parse_pk12(p12, pass, -1, pkey, ocerts)) {
@@ -130,8 +130,9 @@ PKCS12_parse(PKCS12 *p12, const char *pass, EVP_PKEY **pkey, X509 **cert,
 		goto err;
 	}
 
-	while ((x = sk_X509_pop(ocerts))) {
-		if (pkey && *pkey && cert && !*cert) {
+	while ((x = sk_X509_pop(ocerts)) != NULL) {
+		if (pkey != NULL && *pkey != NULL &&
+		    cert != NULL && *cert == NULL) {
 			ERR_set_mark();
 			if (X509_check_private_key(x, *pkey)) {
 				*cert = x;
@@ -140,31 +141,31 @@ PKCS12_parse(PKCS12 *p12, const char *pass, EVP_PKEY **pkey, X509 **cert,
 			ERR_pop_to_mark();
 		}
 
-		if (ca && x) {
-			if (!*ca)
+		if (ca != NULL && x != NULL) {
+			if (*ca == NULL)
 				*ca = sk_X509_new_null();
-			if (!*ca)
+			if (*ca == NULL)
 				goto err;
 			if (!sk_X509_push(*ca, x))
 				goto err;
 			x = NULL;
 		}
 		X509_free(x);
+		x = NULL;
 	}
 
-	if (ocerts)
-		sk_X509_pop_free(ocerts, X509_free);
+	sk_X509_pop_free(ocerts, X509_free);
 
 	return 1;
 
 err:
-	if (pkey && *pkey)
+	if (pkey != NULL)
 		EVP_PKEY_free(*pkey);
-	if (cert)
+	if (cert != NULL)
 		X509_free(*cert);
 	X509_free(x);
-	if (ocerts)
-		sk_X509_pop_free(ocerts, X509_free);
+	sk_X509_pop_free(ocerts, X509_free);
+
 	return 0;
 }
 
@@ -225,14 +226,14 @@ parse_bag(PKCS12_SAFEBAG *bag, const char *pass, int passlen, EVP_PKEY **pkey,
 {
 	PKCS8_PRIV_KEY_INFO *p8;
 	X509 *x509;
-	ASN1_TYPE *attrib;
+	const ASN1_TYPE *attrib;
 	ASN1_BMPSTRING *fname = NULL;
 	ASN1_OCTET_STRING *lkid = NULL;
 
-	if ((attrib = PKCS12_get_attr(bag, NID_friendlyName)))
+	if ((attrib = PKCS12_SAFEBAG_get0_attr(bag, NID_friendlyName)))
 		fname = attrib->value.bmpstring;
 
-	if ((attrib = PKCS12_get_attr(bag, NID_localKeyID)))
+	if ((attrib = PKCS12_SAFEBAG_get0_attr(bag, NID_localKeyID)))
 		lkid = attrib->value.octet_string;
 
 	switch (OBJ_obj2nid(bag->type)) {
@@ -265,7 +266,7 @@ parse_bag(PKCS12_SAFEBAG *bag, const char *pass, int passlen, EVP_PKEY **pkey,
 		}
 		if (fname) {
 			int len, r;
-			unsigned char *data;
+			unsigned char *data = NULL;
 			len = ASN1_STRING_to_UTF8(&data, fname);
 			if (len >= 0) {
 				r = X509_alias_set1(x509, data, len);
