@@ -1,4 +1,4 @@
-/*	$OpenBSD: x509_addr.c,v 1.77 2022/02/04 16:34:15 tb Exp $ */
+/*	$OpenBSD: x509_addr.c,v 1.83 2022/05/25 17:10:30 tb Exp $ */
 /*
  * Contributed to the OpenSSL Project by the American Registry for
  * Internet Numbers ("ARIN").
@@ -73,6 +73,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
+#include "asn1_locl.h"
 #include "bytestring.h"
 #include "x509_lcl.h"
 
@@ -485,7 +486,7 @@ IPAddressFamily_find_in_parent(IPAddrBlocks *parent, IPAddressFamily *child_af)
 {
 	int index;
 
-	sk_IPAddressFamily_set_cmp_func(parent, IPAddressFamily_cmp);
+	(void)sk_IPAddressFamily_set_cmp_func(parent, IPAddressFamily_cmp);
 
 	if ((index = sk_IPAddressFamily_find(parent, child_af)) < 0)
 		return NULL;
@@ -847,44 +848,45 @@ range_should_be_prefix(const unsigned char *min, const unsigned char *max,
 }
 
 /*
- * Construct a prefix.
+ * Fill IPAddressOrRange with bit string encoding of a prefix - RFC 3779, 2.1.1.
  */
 static int
-make_addressPrefix(IPAddressOrRange **result, unsigned char *addr,
-    unsigned int afi, int prefix_len)
+make_addressPrefix(IPAddressOrRange **out_aor, uint8_t *addr, uint32_t afi,
+    int prefix_len)
 {
-	IPAddressOrRange *aor;
-	int afi_len, byte_len, bit_len, max_len;
+	IPAddressOrRange *aor = NULL;
+	int afi_len, max_len, num_bits, num_octets;
+	uint8_t unused_bits;
 
 	if (prefix_len < 0)
-		return 0;
+		goto err;
 
 	max_len = 16;
 	if ((afi_len = length_from_afi(afi)) > 0)
 		max_len = afi_len;
 	if (prefix_len > 8 * max_len)
-		return 0;
+		goto err;
 
-	byte_len = (prefix_len + 7) / 8;
-	bit_len = prefix_len % 8;
+	num_octets = (prefix_len + 7) / 8;
+	num_bits = prefix_len % 8;
+
+	unused_bits = 0;
+	if (num_bits > 0)
+		unused_bits = 8 - num_bits;
 
 	if ((aor = IPAddressOrRange_new()) == NULL)
-		return 0;
+		goto err;
+
 	aor->type = IPAddressOrRange_addressPrefix;
+
 	if ((aor->u.addressPrefix = ASN1_BIT_STRING_new()) == NULL)
 		goto err;
-
-	if (!ASN1_BIT_STRING_set(aor->u.addressPrefix, addr, byte_len))
+	if (!ASN1_BIT_STRING_set(aor->u.addressPrefix, addr, num_octets))
+		goto err;
+	if (!asn1_abs_set_unused_bits(aor->u.addressPrefix, unused_bits))
 		goto err;
 
-	aor->u.addressPrefix->flags &= ~7;
-	aor->u.addressPrefix->flags |= ASN1_STRING_FLAG_BITS_LEFT;
-	if (bit_len > 0) {
-		aor->u.addressPrefix->data[byte_len - 1] &= ~(0xff >> bit_len);
-		aor->u.addressPrefix->flags |= 8 - bit_len;
-	}
-
-	*result = aor;
+	*out_aor = aor;
 	return 1;
 
  err:
@@ -892,59 +894,126 @@ make_addressPrefix(IPAddressOrRange **result, unsigned char *addr,
 	return 0;
 }
 
+static uint8_t
+count_trailing_zeroes(uint8_t octet)
+{
+	uint8_t count = 0;
+
+	if (octet == 0)
+		return 8;
+
+	while ((octet & (1 << count)) == 0)
+		count++;
+
+	return count;
+}
+
+static int
+trim_end_u8(CBS *cbs, uint8_t trim)
+{
+	uint8_t octet;
+
+	while (CBS_len(cbs) > 0) {
+		if (!CBS_peek_last_u8(cbs, &octet))
+			return 0;
+		if (octet != trim)
+			return 1;
+		if (!CBS_get_last_u8(cbs, &octet))
+			return 0;
+	}
+
+	return 1;
+}
+
 /*
- * Construct a range.  If it can be expressed as a prefix,
- * return a prefix instead.  Doing this here simplifies
- * the rest of the code considerably.
+ * Populate IPAddressOrRange with bit string encoding of a range, see
+ * RFC 3779, 2.1.2.
  */
 static int
-make_addressRange(IPAddressOrRange **result, unsigned char *min,
-    unsigned char *max, unsigned int afi, int length)
+make_addressRange(IPAddressOrRange **out_aor, uint8_t *min, uint8_t *max,
+    uint32_t afi, int length)
 {
-	IPAddressOrRange *aor;
-	int i, prefix_len;
+	IPAddressOrRange *aor = NULL;
+	IPAddressRange *range;
+	int prefix_len;
+	CBS cbs;
+	size_t max_len, min_len;
+	uint8_t unused_bits_min, unused_bits_max;
+	uint8_t octet;
 
 	if (memcmp(min, max, length) > 0)
-		return 0;
+		goto err;
+
+	/*
+	 * RFC 3779, 2.2.3.6 - a range that can be expressed as a prefix
+	 * must be encoded as a prefix.
+	 */
 
 	if ((prefix_len = range_should_be_prefix(min, max, length)) >= 0)
-		return make_addressPrefix(result, min, afi, prefix_len);
+		return make_addressPrefix(out_aor, min, afi, prefix_len);
+
+	/*
+	 * The bit string representing min is formed by removing all its
+	 * trailing zero bits, so remove all trailing zero octets and count
+	 * the trailing zero bits of the last octet.
+	 */
+
+	CBS_init(&cbs, min, length);
+
+	if (!trim_end_u8(&cbs, 0x00))
+		goto err;
+
+	unused_bits_min = 0;
+	if ((min_len = CBS_len(&cbs)) > 0) {
+		if (!CBS_peek_last_u8(&cbs, &octet))
+			goto err;
+
+		unused_bits_min = count_trailing_zeroes(octet);
+	}
+
+	/*
+	 * The bit string representing max is formed by removing all its
+	 * trailing one bits, so remove all trailing 0xff octets and count
+	 * the trailing ones of the last octet.
+	 */
+
+	CBS_init(&cbs, max, length);
+
+	if (!trim_end_u8(&cbs, 0xff))
+		goto err;
+
+	unused_bits_max = 0;
+	if ((max_len = CBS_len(&cbs)) > 0) {
+		if (!CBS_peek_last_u8(&cbs, &octet))
+			goto err;
+
+		unused_bits_max = count_trailing_zeroes(octet + 1);
+	}
+
+	/*
+	 * Populate IPAddressOrRange.
+	 */
 
 	if ((aor = IPAddressOrRange_new()) == NULL)
-		return 0;
+		goto err;
+
 	aor->type = IPAddressOrRange_addressRange;
-	if ((aor->u.addressRange = IPAddressRange_new()) == NULL)
+
+	if ((range = aor->u.addressRange = IPAddressRange_new()) == NULL)
 		goto err;
 
-	for (i = length; i > 0 && min[i - 1] == 0x00; --i)
-		continue;
-	if (!ASN1_BIT_STRING_set(aor->u.addressRange->min, min, i))
+	if (!ASN1_BIT_STRING_set(range->min, min, min_len))
 		goto err;
-	aor->u.addressRange->min->flags &= ~7;
-	aor->u.addressRange->min->flags |= ASN1_STRING_FLAG_BITS_LEFT;
-	if (i > 0) {
-		unsigned char b = min[i - 1];
-		int j = 1;
-		while ((b & (0xffU >> j)) != 0)
-			++j;
-		aor->u.addressRange->min->flags |= 8 - j;
-	}
-
-	for (i = length; i > 0 && max[i - 1] == 0xff; --i)
-		continue;
-	if (!ASN1_BIT_STRING_set(aor->u.addressRange->max, max, i))
+	if (!asn1_abs_set_unused_bits(range->min, unused_bits_min))
 		goto err;
-	aor->u.addressRange->max->flags &= ~7;
-	aor->u.addressRange->max->flags |= ASN1_STRING_FLAG_BITS_LEFT;
-	if (i > 0) {
-		unsigned char b = max[i - 1];
-		int j = 1;
-		while ((b & (0xffU >> j)) != (0xffU >> j))
-			++j;
-		aor->u.addressRange->max->flags |= 8 - j;
-	}
 
-	*result = aor;
+	if (!ASN1_BIT_STRING_set(range->max, max, max_len))
+		goto err;
+	if (!asn1_abs_set_unused_bits(range->max, unused_bits_max))
+		goto err;
+
+	*out_aor = aor;
+
 	return 1;
 
  err:
@@ -1049,10 +1118,12 @@ make_prefix_or_range(IPAddrBlocks *addr, const unsigned afi,
 
 	switch (afi) {
 	case IANA_AFI_IPV4:
-		sk_IPAddressOrRange_set_cmp_func(aors, v4IPAddressOrRange_cmp);
+		(void)sk_IPAddressOrRange_set_cmp_func(aors,
+		    v4IPAddressOrRange_cmp);
 		break;
 	case IANA_AFI_IPV6:
-		sk_IPAddressOrRange_set_cmp_func(aors, v6IPAddressOrRange_cmp);
+		(void)sk_IPAddressOrRange_set_cmp_func(aors,
+		    v6IPAddressOrRange_cmp);
 		break;
 	}
 
@@ -1237,10 +1308,6 @@ X509v3_addr_is_canonical(IPAddrBlocks *addr)
 			aor_a = sk_IPAddressOrRange_value(aors, j);
 			aor_b = sk_IPAddressOrRange_value(aors, j + 1);
 
-			/*
-			 * XXX - check that both are either a prefix or a range.
-			 */
-
 			if (!extract_min_max(aor_a, a_min, a_max, length) ||
 			    !extract_min_max(aor_b, b_min, b_max, length))
 				return 0;
@@ -1349,7 +1416,7 @@ IPAddressOrRanges_canonize(IPAddressOrRanges *aors, const unsigned afi)
 		if (!make_addressRange(&merged, a_min, b_max, afi, length))
 			return 0;
 		sk_IPAddressOrRange_set(aors, i, merged);
-		sk_IPAddressOrRange_delete(aors, i + 1);
+		(void)sk_IPAddressOrRange_delete(aors, i + 1);
 		IPAddressOrRange_free(a);
 		IPAddressOrRange_free(b);
 		i--;
@@ -1394,7 +1461,7 @@ X509v3_addr_canonize(IPAddrBlocks *addr)
 			return 0;
 	}
 
-	sk_IPAddressFamily_set_cmp_func(addr, IPAddressFamily_cmp);
+	(void)sk_IPAddressFamily_set_cmp_func(addr, IPAddressFamily_cmp);
 	sk_IPAddressFamily_sort(addr);
 
 	return X509v3_addr_is_canonical(addr);
@@ -1772,17 +1839,17 @@ addr_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509) *chain,
 
 	/*
 	 * Figure out where to start. If we don't have an extension to check,
-	 * we're done.  Otherwise, check canonical form and set up for walking
-	 * up the chain.
+	 * (either extracted from the leaf or passed by the caller), we're done.
+	 * Otherwise, check canonical form and set up for walking up the chain.
 	 */
 	if (ext == NULL) {
 		depth = 0;
 		cert = sk_X509_value(chain, depth);
+		if ((X509_get_extension_flags(cert) & EXFLAG_INVALID) != 0)
+			goto done;
 		if ((ext = cert->rfc3779_addr) == NULL)
 			goto done;
-	}
-
-	if (!X509v3_addr_is_canonical(ext)) {
+	} else if (!X509v3_addr_is_canonical(ext)) {
 		if ((ret = verify_error(ctx, cert,
 		    X509_V_ERR_INVALID_EXTENSION, depth)) == 0)
 			goto done;
@@ -1804,6 +1871,12 @@ addr_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509) *chain,
 	for (depth++; depth < sk_X509_num(chain); depth++) {
 		cert = sk_X509_value(chain, depth);
 
+		if ((X509_get_extension_flags(cert) & EXFLAG_INVALID) != 0) {
+			if ((ret = verify_error(ctx, cert,
+			    X509_V_ERR_INVALID_EXTENSION, depth)) == 0)
+				goto done;
+		}
+
 		if ((parent = cert->rfc3779_addr) == NULL) {
 			for (i = 0; i < sk_IPAddressFamily_num(child); i++) {
 				child_af = sk_IPAddressFamily_value(child, i);
@@ -1820,12 +1893,6 @@ addr_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509) *chain,
 			continue;
 		}
 
-		if (!X509v3_addr_is_canonical(parent)) {
-			if ((ret = verify_error(ctx, cert,
-			    X509_V_ERR_INVALID_EXTENSION, depth)) == 0)
-				goto done;
-		}
-
 		/*
 		 * Check that the child's resources are covered by the parent.
 		 * Each covered resource is replaced with the parent's resource
