@@ -1,17 +1,65 @@
+/*	$OpenBSD: x509_asid.c,v 1.35 2022/07/30 17:50:17 tb Exp $ */
 /*
- * Copyright 2006-2018 The OpenSSL Project Authors. All Rights Reserved.
+ * Contributed to the OpenSSL Project by the American Registry for
+ * Internet Numbers ("ARIN").
+ */
+/* ====================================================================
+ * Copyright (c) 2006-2018 The OpenSSL Project.  All rights reserved.
  *
- * Licensed under the OpenSSL license (the "License").  You may not use
- * this file except in compliance with the License.  You can obtain a copy
- * in the file LICENSE in the source distribution or at
- * https://www.openssl.org/source/license.html
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. All advertising materials mentioning features or use of this
+ *    software must display the following acknowledgment:
+ *    "This product includes software developed by the OpenSSL Project
+ *    for use in the OpenSSL Toolkit. (http://www.OpenSSL.org/)"
+ *
+ * 4. The names "OpenSSL Toolkit" and "OpenSSL Project" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For written permission, please contact
+ *    licensing@OpenSSL.org.
+ *
+ * 5. Products derived from this software may not be called "OpenSSL"
+ *    nor may "OpenSSL" appear in their names without prior written
+ *    permission of the OpenSSL Project.
+ *
+ * 6. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by the OpenSSL Project
+ *    for use in the OpenSSL Toolkit (http://www.OpenSSL.org/)"
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE OpenSSL PROJECT ``AS IS'' AND ANY
+ * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE OpenSSL PROJECT OR
+ * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+ * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+ * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * ====================================================================
+ *
+ * This product includes cryptographic software written by Eric Young
+ * (eay@cryptsoft.com).  This product includes software written by Tim
+ * Hudson (tjh@cryptsoft.com).
  */
 
 /*
  * Implementation of RFC 3779 section 3.2.
  */
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -25,7 +73,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
-#include "ext_dat.h"
+#include "x509_lcl.h"
 
 #ifndef OPENSSL_NO_RFC3779
 
@@ -312,7 +360,7 @@ ASIdOrRange_cmp(const ASIdOrRange *const *a_, const ASIdOrRange *const *b_)
 {
 	const ASIdOrRange *a = *a_, *b = *b_;
 
-    /* XXX: these asserts need to be replaced */
+	/* XXX: these asserts need to be replaced */
 	OPENSSL_assert((a->type == ASIdOrRange_id && a->u.id != NULL) ||
 	    (a->type == ASIdOrRange_range && a->u.range != NULL &&
 	     a->u.range->min != NULL && a->u.range->max != NULL));
@@ -425,8 +473,6 @@ X509v3_asid_add_id_or_range(ASIdentifiers *asid, int which, ASN1_INTEGER *min,
 static int
 extract_min_max(ASIdOrRange *aor, ASN1_INTEGER **min, ASN1_INTEGER **max)
 {
-	OPENSSL_assert(aor != NULL);
-
 	switch (aor->type) {
 	case ASIdOrRange_id:
 		*min = aor->u.id;
@@ -602,7 +648,8 @@ ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 		/*
 		 * Make sure we're properly sorted (paranoia).
 		 */
-		OPENSSL_assert(ASN1_INTEGER_cmp(a_min, b_min) <= 0);
+		if (ASN1_INTEGER_cmp(a_min, b_min) > 0)
+			goto done;
 
 		/*
 		 * Punt inverted ranges.
@@ -689,7 +736,8 @@ ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 	}
 
 	/* Paranoia */
-	OPENSSL_assert(ASIdentifierChoice_is_canonical(choice));
+	if (!ASIdentifierChoice_is_canonical(choice))
+		goto done;
 
 	ret = 1;
 
@@ -705,9 +753,13 @@ ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 int
 X509v3_asid_canonize(ASIdentifiers *asid)
 {
-	return (asid == NULL ||
-	    (ASIdentifierChoice_canonize(asid->asnum) &&
-	     ASIdentifierChoice_canonize(asid->rdi)));
+	if (asid == NULL)
+		return 1;
+
+	if (!ASIdentifierChoice_canonize(asid->asnum))
+		return 0;
+
+	return ASIdentifierChoice_canonize(asid->rdi);
 }
 
 /*
@@ -830,17 +882,20 @@ v2i_ASIdentifiers(const struct v3_ext_method *method, struct v3_ext_ctx *ctx,
  * OpenSSL dispatch.
  */
 const X509V3_EXT_METHOD v3_asid = {
-	NID_sbgp_autonomousSysNum,  /* nid */
-	0,                          /* flags */
-	&ASIdentifiers_it,          /* template */
-	0, 0, 0, 0,                 /* old functions, ignored */
-	0,                          /* i2s */
-	0,                          /* s2i */
-	0,                          /* i2v */
-	v2i_ASIdentifiers,          /* v2i */
-	i2r_ASIdentifiers,          /* i2r */
-	0,                          /* r2i */
-	NULL                        /* extension-specific data */
+	.ext_nid = NID_sbgp_autonomousSysNum,
+	.ext_flags = 0,
+	.it = &ASIdentifiers_it,
+	.ext_new = NULL,
+	.ext_free = NULL,
+	.d2i = NULL,
+	.i2d = NULL,
+	.i2s = NULL,
+	.s2i = NULL,
+	.i2v = NULL,
+	.v2i = v2i_ASIdentifiers,
+	.i2r = i2r_ASIdentifiers,
+	.r2i = NULL,
+	.usr_data = NULL,
 };
 
 /*
@@ -849,11 +904,20 @@ const X509V3_EXT_METHOD v3_asid = {
 int
 X509v3_asid_inherits(ASIdentifiers *asid)
 {
-	return (asid != NULL &&
-	    ((asid->asnum != NULL &&
-	      asid->asnum->type == ASIdentifierChoice_inherit) ||
-	     (asid->rdi != NULL &&
-	      asid->rdi->type == ASIdentifierChoice_inherit)));
+	if (asid == NULL)
+		return 0;
+
+	if (asid->asnum != NULL) {
+		if (asid->asnum->type == ASIdentifierChoice_inherit)
+			return 1;
+	}
+
+	if (asid->rdi != NULL) {
+		if (asid->rdi->type == ASIdentifierChoice_inherit)
+			return 1;
+	}
+
+	return 0;
 }
 
 /*
@@ -867,6 +931,7 @@ asid_contains(ASIdOrRanges *parent, ASIdOrRanges *child)
 
 	if (child == NULL || parent == child)
 		return 1;
+
 	if (parent == NULL)
 		return 0;
 
@@ -893,20 +958,39 @@ asid_contains(ASIdOrRanges *parent, ASIdOrRanges *child)
 }
 
 /*
- * Test whether a is a subset of b.
+ * Test whether child is a subset of parent.
  */
 int
-X509v3_asid_subset(ASIdentifiers *a, ASIdentifiers *b)
+X509v3_asid_subset(ASIdentifiers *child, ASIdentifiers *parent)
 {
-	return (a == NULL ||
-	    a == b ||
-	    (b != NULL &&
-	     !X509v3_asid_inherits(a) &&
-	     !X509v3_asid_inherits(b) &&
-	     asid_contains(b->asnum->u.asIdsOrRanges,
-	     a->asnum->u.asIdsOrRanges) &&
-	     asid_contains(b->rdi->u.asIdsOrRanges,
-	     a->rdi->u.asIdsOrRanges)));
+	if (child == NULL || child == parent)
+		return 1;
+
+	if (parent == NULL)
+		return 0;
+
+	if (X509v3_asid_inherits(child) || X509v3_asid_inherits(parent))
+		return 0;
+
+	if (child->asnum != NULL) {
+		if (parent->asnum == NULL)
+			return 0;
+
+		if (!asid_contains(parent->asnum->u.asIdsOrRanges,
+		    child->asnum->u.asIdsOrRanges))
+			return 0;
+	}
+
+	if (child->rdi != NULL) {
+		if (parent->rdi == NULL)
+			return 0;
+
+		if (!asid_contains(parent->rdi->u.asIdsOrRanges,
+		    child->rdi->u.asIdsOrRanges))
+			return 0;
+	}
+
+	return 1;
 }
 
 /*
@@ -930,33 +1014,41 @@ X509v3_asid_subset(ASIdentifiers *a, ASIdentifiers *b)
  * Core code for RFC 3779 3.3 path validation.
  */
 static int
-asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509)*chain,
+asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509) *chain,
     ASIdentifiers *ext)
 {
 	ASIdOrRanges *child_as = NULL, *child_rdi = NULL;
 	int i, ret = 1, inherit_as = 0, inherit_rdi = 0;
 	X509 *x;
 
-	OPENSSL_assert(chain != NULL && sk_X509_num(chain) > 0);
-	OPENSSL_assert(ctx != NULL || ext != NULL);
-	OPENSSL_assert(ctx == NULL || ctx->verify_cb != NULL);
+	/* We need a non-empty chain to test against. */
+	if (sk_X509_num(chain) <= 0)
+		goto err;
+	/* We need either a store ctx or an extension to work with. */
+	if (ctx == NULL && ext == NULL)
+		goto err;
+	/* If there is a store ctx, it needs a verify_cb. */
+	if (ctx != NULL && ctx->verify_cb == NULL)
+		goto err;
 
 	/*
-	 * Figure out where to start.  If we don't have an extension to
-	 * check, we're done.  Otherwise, check canonical form and
-	 * set up for walking up the chain.
+	 * Figure out where to start. If we don't have an extension to check,
+	 * (either extracted from the leaf or passed by the caller), we're done.
+	 * Otherwise, check canonical form and set up for walking up the chain.
 	 */
 	if (ext != NULL) {
 		i = -1;
 		x = NULL;
+		if (!X509v3_asid_is_canonical(ext))
+			validation_err(X509_V_ERR_INVALID_EXTENSION);
 	} else {
 		i = 0;
 		x = sk_X509_value(chain, i);
+		if ((X509_get_extension_flags(x) & EXFLAG_INVALID) != 0)
+			goto done;
 		if ((ext = x->rfc3779_asid) == NULL)
 			goto done;
 	}
-	if (!X509v3_asid_is_canonical(ext))
-		validation_err(X509_V_ERR_INVALID_EXTENSION);
 	if (ext->asnum != NULL) {
 		switch (ext->asnum->type) {
 		case ASIdentifierChoice_inherit:
@@ -984,15 +1076,14 @@ asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509)*chain,
 	 */
 	for (i++; i < sk_X509_num(chain); i++) {
 		x = sk_X509_value(chain, i);
-		OPENSSL_assert(x != NULL);
 
+		if ((X509_get_extension_flags(x) & EXFLAG_INVALID) != 0)
+			validation_err(X509_V_ERR_INVALID_EXTENSION);
 		if (x->rfc3779_asid == NULL) {
 			if (child_as != NULL || child_rdi != NULL)
 				validation_err(X509_V_ERR_UNNESTED_RESOURCE);
 			continue;
 		}
-		if (!X509v3_asid_is_canonical(x->rfc3779_asid))
-			validation_err(X509_V_ERR_INVALID_EXTENSION);
 		if (x->rfc3779_asid->asnum == NULL && child_as != NULL) {
 			validation_err(X509_V_ERR_UNNESTED_RESOURCE);
 			child_as = NULL;
@@ -1031,7 +1122,9 @@ asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509)*chain,
 	/*
 	 * Trust anchor can't inherit.
 	 */
-	OPENSSL_assert(x != NULL);
+
+	if (x == NULL)
+		goto err;
 
 	if (x->rfc3779_asid != NULL) {
 		if (x->rfc3779_asid->asnum != NULL &&
@@ -1044,6 +1137,12 @@ asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509)*chain,
 
  done:
 	return ret;
+
+ err:
+	if (ctx != NULL)
+		ctx->error = X509_V_ERR_UNSPECIFIED;
+
+	return 0;
 }
 
 #undef validation_err
@@ -1054,9 +1153,7 @@ asid_validate_path_internal(X509_STORE_CTX *ctx, STACK_OF(X509)*chain,
 int
 X509v3_asid_validate_path(X509_STORE_CTX *ctx)
 {
-	if (ctx->chain == NULL ||
-	    sk_X509_num(ctx->chain) == 0 ||
-	    ctx->verify_cb == NULL) {
+	if (sk_X509_num(ctx->chain) <= 0 || ctx->verify_cb == NULL) {
 		ctx->error = X509_V_ERR_UNSPECIFIED;
 		return 0;
 	}
@@ -1068,12 +1165,12 @@ X509v3_asid_validate_path(X509_STORE_CTX *ctx)
  * Test whether chain covers extension.
  */
 int
-X509v3_asid_validate_resource_set(STACK_OF(X509)*chain, ASIdentifiers *ext,
+X509v3_asid_validate_resource_set(STACK_OF(X509) *chain, ASIdentifiers *ext,
     int allow_inheritance)
 {
 	if (ext == NULL)
 		return 1;
-	if (chain == NULL || sk_X509_num(chain) == 0)
+	if (sk_X509_num(chain) <= 0)
 		return 0;
 	if (!allow_inheritance && X509v3_asid_inherits(ext))
 		return 0;
