@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_rsa.c,v 1.34 2021/06/11 11:13:53 jsing Exp $ */
+/* $OpenBSD: ssl_rsa.c,v 1.48 2022/08/31 20:49:37 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -66,12 +66,13 @@
 
 #include "ssl_locl.h"
 
-static int ssl_set_cert(CERT *c, X509 *x509);
-static int ssl_set_pkey(CERT *c, EVP_PKEY *pkey);
-static int use_certificate_chain_bio(BIO *in, CERT *cert,
-    pem_password_cb *passwd_cb, void *passwd_arg);
-static int use_certificate_chain_file(const char *file, CERT *cert,
-    pem_password_cb *passwd_cb, void *passwd_arg);
+static int ssl_get_password_cb_and_arg(SSL_CTX *ctx, SSL *ssl,
+    pem_password_cb **passwd_cb, void **passwd_arg);
+static int ssl_set_cert(SSL_CTX *ctx, SSL *ssl, X509 *x509);
+static int ssl_set_pkey(SSL_CTX *ctx, SSL *ssl, EVP_PKEY *pkey);
+static int ssl_use_certificate_chain_bio(SSL_CTX *ctx, SSL *ssl, BIO *in);
+static int ssl_use_certificate_chain_file(SSL_CTX *ctx, SSL *ssl,
+    const char *file);
 
 int
 SSL_use_certificate(SSL *ssl, X509 *x)
@@ -80,7 +81,7 @@ SSL_use_certificate(SSL *ssl, X509 *x)
 		SSLerror(ssl, ERR_R_PASSED_NULL_PARAMETER);
 		return (0);
 	}
-	return (ssl_set_cert(ssl->cert, x));
+	return ssl_set_cert(NULL, ssl, x);
 }
 
 int
@@ -91,7 +92,7 @@ SSL_use_certificate_file(SSL *ssl, const char *file, int type)
 	int ret = 0;
 	X509 *x = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerror(ssl, ERR_R_BUF_LIB);
 		goto end;
@@ -161,51 +162,62 @@ SSL_use_RSAPrivateKey(SSL *ssl, RSA *rsa)
 	RSA_up_ref(rsa);
 	EVP_PKEY_assign_RSA(pkey, rsa);
 
-	ret = ssl_set_pkey(ssl->cert, pkey);
+	ret = ssl_set_pkey(NULL, ssl, pkey);
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
 
 static int
-ssl_set_pkey(CERT *c, EVP_PKEY *pkey)
+ssl_set_pkey(SSL_CTX *ctx, SSL *ssl, EVP_PKEY *pkey)
 {
+	SSL_CERT *c;
 	int i;
 
-	i = ssl_cert_type(NULL, pkey);
+	i = ssl_cert_type(pkey);
 	if (i < 0) {
 		SSLerrorx(SSL_R_UNKNOWN_CERTIFICATE_TYPE);
 		return (0);
 	}
 
+	if ((c = ssl_get0_cert(ctx, ssl)) == NULL)
+		return (0);
+
 	if (c->pkeys[i].x509 != NULL) {
 		EVP_PKEY *pktmp;
-		pktmp = X509_get_pubkey(c->pkeys[i].x509);
+
+		if ((pktmp = X509_get0_pubkey(c->pkeys[i].x509)) == NULL)
+			return 0;
+
+		/*
+		 * Callers of EVP_PKEY_copy_parameters() can't distinguish
+		 * errors from the absence of a param_copy() method. So
+		 * pretend it can never fail.
+		 */
 		EVP_PKEY_copy_parameters(pktmp, pkey);
-		EVP_PKEY_free(pktmp);
+
 		ERR_clear_error();
 
 		/*
 		 * Don't check the public/private key, this is mostly
 		 * for smart cards.
 		 */
-		if ((pkey->type == EVP_PKEY_RSA) &&
-			(RSA_flags(pkey->pkey.rsa) & RSA_METHOD_FLAG_NO_CHECK))
-;
-		else
-		if (!X509_check_private_key(c->pkeys[i].x509, pkey)) {
-			X509_free(c->pkeys[i].x509);
-			c->pkeys[i].x509 = NULL;
-			return 0;
+		if (EVP_PKEY_id(pkey) != EVP_PKEY_RSA ||
+		    !(RSA_flags(EVP_PKEY_get0_RSA(pkey)) & RSA_METHOD_FLAG_NO_CHECK)) {
+			if (!X509_check_private_key(c->pkeys[i].x509, pkey)) {
+				X509_free(c->pkeys[i].x509);
+				c->pkeys[i].x509 = NULL;
+				return 0;
+			}
 		}
 	}
 
 	EVP_PKEY_free(c->pkeys[i].privatekey);
-	CRYPTO_add(&pkey->references, 1, CRYPTO_LOCK_EVP_PKEY);
+	EVP_PKEY_up_ref(pkey);
 	c->pkeys[i].privatekey = pkey;
 	c->key = &(c->pkeys[i]);
 
 	c->valid = 0;
-	return (1);
+	return 1;
 }
 
 int
@@ -215,7 +227,7 @@ SSL_use_RSAPrivateKey_file(SSL *ssl, const char *file, int type)
 	BIO *in;
 	RSA *rsa = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerror(ssl, ERR_R_BUF_LIB);
 		goto end;
@@ -273,7 +285,7 @@ SSL_use_PrivateKey(SSL *ssl, EVP_PKEY *pkey)
 		SSLerror(ssl, ERR_R_PASSED_NULL_PARAMETER);
 		return (0);
 	}
-	ret = ssl_set_pkey(ssl->cert, pkey);
+	ret = ssl_set_pkey(NULL, ssl, pkey);
 	return (ret);
 }
 
@@ -284,7 +296,7 @@ SSL_use_PrivateKey_file(SSL *ssl, const char *file, int type)
 	BIO *in;
 	EVP_PKEY *pkey = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerror(ssl, ERR_R_BUF_LIB);
 		goto end;
@@ -340,22 +352,45 @@ SSL_CTX_use_certificate(SSL_CTX *ctx, X509 *x)
 		SSLerrorx(ERR_R_PASSED_NULL_PARAMETER);
 		return (0);
 	}
-	return (ssl_set_cert(ctx->internal->cert, x));
+	return ssl_set_cert(ctx, NULL, x);
 }
 
 static int
-ssl_set_cert(CERT *c, X509 *x)
+ssl_get_password_cb_and_arg(SSL_CTX *ctx, SSL *ssl,
+    pem_password_cb **passwd_cb, void **passwd_arg)
 {
+	if (ssl != NULL)
+		ctx = ssl->ctx;
+
+	*passwd_cb = ctx->default_passwd_callback;
+	*passwd_arg = ctx->default_passwd_callback_userdata;
+
+	return 1;
+}
+
+static int
+ssl_set_cert(SSL_CTX *ctx, SSL *ssl, X509 *x)
+{
+	SSL_CERT *c;
 	EVP_PKEY *pkey;
+	int ssl_err;
 	int i;
 
+	if (!ssl_security_cert(ctx, ssl, x, 1, &ssl_err)) {
+		SSLerrorx(ssl_err);
+		return (0);
+	}
+
+	if ((c = ssl_get0_cert(ctx, ssl)) == NULL)
+		return (0);
+
 	pkey = X509_get_pubkey(x);
 	if (pkey == NULL) {
 		SSLerrorx(SSL_R_X509_LIB);
 		return (0);
 	}
 
-	i = ssl_cert_type(x, pkey);
+	i = ssl_cert_type(pkey);
 	if (i < 0) {
 		SSLerrorx(SSL_R_UNKNOWN_CERTIFICATE_TYPE);
 		EVP_PKEY_free(pkey);
@@ -363,36 +398,35 @@ ssl_set_cert(CERT *c, X509 *x)
 	}
 
 	if (c->pkeys[i].privatekey != NULL) {
-		EVP_PKEY_copy_parameters(pkey, c->pkeys[i].privatekey);
+		EVP_PKEY *priv_key = c->pkeys[i].privatekey;
+
+		EVP_PKEY_copy_parameters(pkey, priv_key);
 		ERR_clear_error();
 
 		/*
 		 * Don't check the public/private key, this is mostly
 		 * for smart cards.
 		 */
-		if ((c->pkeys[i].privatekey->type == EVP_PKEY_RSA) &&
-			(RSA_flags(c->pkeys[i].privatekey->pkey.rsa) &
-		RSA_METHOD_FLAG_NO_CHECK))
-;
-		else
-		if (!X509_check_private_key(x, c->pkeys[i].privatekey)) {
-			/*
-			 * don't fail for a cert/key mismatch, just free
-			 * current private key (when switching to a different
-			 * cert & key, first this function should be used,
-			 * then ssl_set_pkey
-			 */
-			EVP_PKEY_free(c->pkeys[i].privatekey);
-			c->pkeys[i].privatekey = NULL;
-			/* clear error queue */
-			ERR_clear_error();
+		if (EVP_PKEY_id(priv_key) != EVP_PKEY_RSA ||
+		    !(RSA_flags(EVP_PKEY_get0_RSA(priv_key)) & RSA_METHOD_FLAG_NO_CHECK)) {
+			if (!X509_check_private_key(x, priv_key)) {
+				/*
+				 * don't fail for a cert/key mismatch, just free
+				 * current private key (when switching to a
+				 * different cert & key, first this function
+				 * should be used, then ssl_set_pkey.
+				 */
+				EVP_PKEY_free(c->pkeys[i].privatekey);
+				c->pkeys[i].privatekey = NULL;
+				ERR_clear_error();
+			}
 		}
 	}
 
 	EVP_PKEY_free(pkey);
 
 	X509_free(c->pkeys[i].x509);
-	CRYPTO_add(&x->references, 1, CRYPTO_LOCK_X509);
+	X509_up_ref(x);
 	c->pkeys[i].x509 = x;
 	c->key = &(c->pkeys[i]);
 
@@ -408,7 +442,7 @@ SSL_CTX_use_certificate_file(SSL_CTX *ctx, const char *file, int type)
 	int ret = 0;
 	X509 *x = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerrorx(ERR_R_BUF_LIB);
 		goto end;
@@ -477,7 +511,7 @@ SSL_CTX_use_RSAPrivateKey(SSL_CTX *ctx, RSA *rsa)
 	RSA_up_ref(rsa);
 	EVP_PKEY_assign_RSA(pkey, rsa);
 
-	ret = ssl_set_pkey(ctx->internal->cert, pkey);
+	ret = ssl_set_pkey(ctx, NULL, pkey);
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
@@ -489,7 +523,7 @@ SSL_CTX_use_RSAPrivateKey_file(SSL_CTX *ctx, const char *file, int type)
 	BIO *in;
 	RSA *rsa = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerrorx(ERR_R_BUF_LIB);
 		goto end;
@@ -545,7 +579,7 @@ SSL_CTX_use_PrivateKey(SSL_CTX *ctx, EVP_PKEY *pkey)
 		SSLerrorx(ERR_R_PASSED_NULL_PARAMETER);
 		return (0);
 	}
-	return (ssl_set_pkey(ctx->internal->cert, pkey));
+	return ssl_set_pkey(ctx, NULL, pkey);
 }
 
 int
@@ -555,7 +589,7 @@ SSL_CTX_use_PrivateKey_file(SSL_CTX *ctx, const char *file, int type)
 	BIO *in;
 	EVP_PKEY *pkey = NULL;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerrorx(ERR_R_BUF_LIB);
 		goto end;
@@ -612,29 +646,33 @@ SSL_CTX_use_PrivateKey_ASN1(int type, SSL_CTX *ctx, const unsigned char *d,
  * sent to the peer in the Certificate message.
  */
 static int
-use_certificate_chain_bio(BIO *in, CERT *cert, pem_password_cb *passwd_cb,
-    void *passwd_arg)
+ssl_use_certificate_chain_bio(SSL_CTX *ctx, SSL *ssl, BIO *in)
 {
+	pem_password_cb *passwd_cb;
+	void *passwd_arg;
 	X509 *ca, *x = NULL;
 	unsigned long err;
 	int ret = 0;
 
+	if (!ssl_get_password_cb_and_arg(ctx, ssl, &passwd_cb, &passwd_arg))
+		goto err;
+
 	if ((x = PEM_read_bio_X509_AUX(in, NULL, passwd_cb, passwd_arg)) ==
 	    NULL) {
 		SSLerrorx(ERR_R_PEM_LIB);
 		goto err;
 	}
 
-	if (!ssl_set_cert(cert, x))
+	if (!ssl_set_cert(ctx, ssl, x))
 		goto err;
 
-	if (!ssl_cert_set0_chain(cert, NULL))
+	if (!ssl_cert_set0_chain(ctx, ssl, NULL))
 		goto err;
 
 	/* Process any additional CA certificates. */
 	while ((ca = PEM_read_bio_X509(in, NULL, passwd_cb, passwd_arg)) !=
 	    NULL) {
-		if (!ssl_cert_add0_chain_cert(cert, ca)) {
+		if (!ssl_cert_add0_chain_cert(ctx, ssl, ca)) {
 			X509_free(ca);
 			goto err;
 		}
@@ -655,13 +693,12 @@ use_certificate_chain_bio(BIO *in, CERT *cert, pem_password_cb *passwd_cb,
 }
 
 int
-use_certificate_chain_file(const char *file, CERT *cert,
-    pem_password_cb *passwd_cb, void *passwd_arg)
+ssl_use_certificate_chain_file(SSL_CTX *ctx, SSL *ssl, const char *file)
 {
 	BIO *in;
 	int ret = 0;
 
-	in = BIO_new(BIO_s_file_internal());
+	in = BIO_new(BIO_s_file());
 	if (in == NULL) {
 		SSLerrorx(ERR_R_BUF_LIB);
 		goto end;
@@ -672,7 +709,7 @@ use_certificate_chain_file(const char *file, CERT *cert,
 		goto end;
 	}
 
-	ret = use_certificate_chain_bio(in, cert, passwd_cb, passwd_arg);
+	ret = ssl_use_certificate_chain_bio(ctx, ssl, in);
 
  end:
 	BIO_free(in);
@@ -682,17 +719,13 @@ use_certificate_chain_file(const char *file, CERT *cert,
 int
 SSL_CTX_use_certificate_chain_file(SSL_CTX *ctx, const char *file)
 {
-	return use_certificate_chain_file(file, ctx->internal->cert,
-	    ctx->default_passwd_callback,
-	    ctx->default_passwd_callback_userdata);
+	return ssl_use_certificate_chain_file(ctx, NULL, file);
 }
 
 int
 SSL_use_certificate_chain_file(SSL *ssl, const char *file)
 {
-	return use_certificate_chain_file(file, ssl->cert,
-	    ssl->ctx->default_passwd_callback,
-	    ssl->ctx->default_passwd_callback_userdata);
+	return ssl_use_certificate_chain_file(NULL, ssl, file);
 }
 
 int
@@ -707,9 +740,7 @@ SSL_CTX_use_certificate_chain_mem(SSL_CTX *ctx, void *buf, int len)
 		goto end;
 	}
 
-	ret = use_certificate_chain_bio(in, ctx->internal->cert,
-	    ctx->default_passwd_callback,
-	    ctx->default_passwd_callback_userdata);
+	ret = ssl_use_certificate_chain_bio(ctx, NULL, in);
 
  end:
 	BIO_free(in);
